<?php
/*
Plugin Name: EZ Optimize Media Optimizer
Plugin URI: http://ezoptimize.com
Description: Automatically optimize all your media to get the minimum possible size without affecting the quality of the images. The easiest way to take care of Google PageSpeed "Optimize images" notification
Version: 2.0.1
Author: The Mediashop
Author URI: https://themediashop.co
License: GPL2
*/

class EZO_EZOptimize
{
    private static $instance = null;
    private $plugin_path;
    private $plugin_url;
    private $text_domain = '';
    private $menuItem;

    /**
     * Creates or returns an instance of this class.
     */
    public static function get_instance()
    {
        // If an instance hasn't been created and set to $instance create an instance and set it to $instance.
        if (null == self::$instance) {
            self::$instance = new self;
        }
        return self::$instance;
    }

    public function isJSON($string)
    {
        return is_string($string) && is_array(json_decode($string, true)) ? true : false;
    }

    /**
     * Initializes the plugin by setting localization, hooks, filters, and administrative functions.
     */
    private function __construct()
    {
        $this->plugin_path = plugin_dir_path(__FILE__);
        $this->plugin_url = plugin_dir_url(__FILE__);

        global $wpdb;
        load_plugin_textdomain($this->text_domain, false, 'lang');
        register_activation_hook(__FILE__, function () {
            if (!get_option('ezoptimize_key')) {
                update_option('ezoptimize_key', false);
            }
            //Go through media and optimize everything
            if (is_multisite()) {
                //Add the plugin interface with the VueJS
            } else {
                if (!get_option('ezoptimize')) {
                    //Create queue
                    $filesStorage = $this->getFiles();
                    update_option('ezoptimize', (object)['queue' => $filesStorage, 'optimized' => []]);
                }
            }
        });
        if (isset($_GET['flush'])) {
            $filesStorage = $this->getFiles();
            update_option('ezoptimize', (object)['queue' => $filesStorage, 'optimized' => []]);
        }

        add_action('add_attachment', function () {
            $ezOptimize = get_option('ezoptimize', true);
            $filesStorage = $this->getFiles();
            $ezOptimize->queue = $filesStorage;
            update_option('ezoptimize', $ezOptimize);
        });

        //Load assets
        add_action('wp_enqueue_scripts', [$this, 'loadAssets']);
        add_action('admin_enqueue_scripts', [$this, 'loadAssets']);

        //Ajax optimize
        add_action('wp_ajax_ez_optimize', function () {
            $ezOptimize = get_option('ezoptimize', true);

            $apiKey = filter_input(INPUT_POST, 'apikey', FILTER_SANITIZE_STRING);
            $path = filter_input(INPUT_POST, 'path', FILTER_SANITIZE_STRING);
            echo json_encode($this->ezOptimize($apiKey, $path));
            wp_die();
        });
        //Ajax revert
        add_action('wp_ajax_ez_revert', function () {
            $ezOptimize = get_option('ezoptimize', true);
            $path = filter_input(INPUT_POST, 'path', FILTER_SANITIZE_STRING);
            if (file_exists($path . '.uncompressed')) {
                //Delete
                unlink($path);
                //Revert
                rename($path . '.uncompressed', $path);
            }
            $queue = $this->reindexQueue();
            $queue->state = 'success';
            echo json_encode($queue);
            wp_die();
        });
        //Ajax save api key
        add_action('wp_ajax_ez_api_key', function () {
            $apiKey = filter_input(INPUT_POST, 'apikey', FILTER_SANITIZE_STRING);
            $auto = filter_input(INPUT_POST, 'auto', FILTER_SANITIZE_STRING);
            update_option('ezoptimize_key', $apiKey);
            update_option('ezoptimize_auto', $auto == 'true' ? true : false);
            echo json_encode(['state' => 'successful']);
            wp_die();
        });

        //Optimize new uploads automatically
        add_filter('wp_generate_attachment_metadata', function ($upload) {
            $ezOptimize = get_option('ezoptimize', true);
            $queue = $this->reindexQueue();
            $ezOptimize->queue = $queue->queue;
            $ezOptimize->optimized = $queue->optimized;
            update_option('ezoptimize', $ezOptimize);
            if (get_option('ezoptimize_auto', false)) {
                $apiKey = get_option('ezoptimize_key');
                foreach ($ezOptimize->queue as $item) {
                    $this->ezOptimize($apiKey, $item->path);
                }
                $queue = $this->reindexQueue();
                $ezOptimize->queue = $queue->queue;
                $ezOptimize->optimized = $queue->optimized;
                update_option('ezoptimize', $ezOptimize);
            }
            return $upload;
        });

        add_action('admin_menu', function () {
            $this->menuItem = add_menu_page(
                'EZOptimize', 'EZOptimize', 'manage_options', 'ezoptimize-menu', function () {
                if (isset($_GET['reindex'])) {
                    $this->reindexQueue();
                }
                //Menu Page
                $ezOptimize = get_option('ezoptimize', []);
                $apiKey = get_option('ezoptimize_key', '');
                $autoOptimize = get_option('ezoptimize_auto', false);

                $ezOptimize->apikey = $apiKey;
                $ezOptimize->auto = $autoOptimize;

                $vueData = json_encode($ezOptimize);
                ?>
                <script>
                    window.vueData = <?= $vueData; ?>;
                </script>
                <div class="wrap">
                    <h1>EZOptimize by <span><a href="https://themediashop.co" target="_blank">The Mediashop</a></span>
                    </h1>
                    <div id="ezOptimize">
                        <h3>Settings</h3>
                        <p>
                            EZOptimize works by sending your images for processing to our servers, using the algorithms
                            we've perfected over the years. For a free API key register at <a
                                    href="https://ezoptimize.com" target="_blank">EZOptimize.com</a> and start
                            optimizing.
                        </p>
                        <table cellspacing="0" class="akismet-settings">
                            <tbody>
                            <tr>
                                <th class="akismet-api-key" width="10%" align="left" scope="row">API Key</th>
                                <td width="5%">
                                </td>
                                <td align="left">
    										<span class="api-key">
                                                <input id="key" name="key" type="text" size="15"
                                                       class="regular-text code active" v-model="apikey">
                                            </span>
                                </td>
                            </tr>
                            <tr>
                                <th class="akismet-api-key" width="10%" align="left" scope="row">Automatically optimize
                                    new uploads
                                </th>
                                <td width="5%">
                                </td>
                                <td align="left">
                                    <label for="autoptimize_images">
                                        <input name="autoptimize_images" id="autoptimize_images" type="checkbox"
                                               value="1" v-model="auto">
                                        New uploads will be optimized automatically
                                    </label>
                                </td>
                            </tr>
                            <tr>
                                <td></td>
                                <td width="5%"></td>
                                <td>
                                    <button v-on:click.stop="saveApiKey">Save</button>
                                </td>
                            </tr>
                            </tbody>
                        </table>
                        <hr>
                        <h3>Queue - The following {{ queue.length }} images need optimizing => <a
                                    v-on:click.stop="optimizeAll" v-if="!disableButtons">Optimize all</a><span
                                    v-if="disableButtons">Working...</span></h3>
                        <table class="responstable">
                            <tr>
                                <th>Path</th>
                                <th data-th="Driver details"><span>Name</span></th>
                                <th>Size</th>
                                <th>Actions</th>
                            </tr>

                            <tr v-for="item in queue">
                                <td><a v-bind:href="item.url" target="_blank">{{item.url}}</a></td>
                                <td>{{item.name}}</td>
                                <td>{{item.size.toFixed(1)}}kB</td>
                                <td><a v-bind:data-path="item.path" v-on:click.stop="optimizeSingle"
                                       v-if="!disableButtons">Optimize</a><span v-if="disableButtons">Working...</span>
                                </td>
                            </tr>
                        </table>
                        <hr>
                        <h3>History of optimized images</h3>
                        <table class="responstable">
                            <tr>
                                <th>Path</th>
                                <th data-th="Driver details"><span>Name</span></th>
                                <th>Current Size</th>
                                <th>Unoptimized Size</th>
                                <th>Actions</th>
                            </tr>

                            <tr v-for="item in optimized">
                                <td><a v-bind:href="item.url" target="_blank">{{item.url}}</a></td>
                                <td>{{item.name}}</td>
                                <td>{{item.sizeAfter.toFixed(1)}}kB</td>
                                <td>{{item.size.toFixed(1)}}kB</td>
                                <td><a v-bind:data-path="item.path" v-on:click.stop="revert" v-if="!disableButtons">Revert</a><span
                                            v-if="disableButtons">Working...</span></td>
                            </tr>
                        </table>
                    </div>
                </div>
                <?php
            }, 'dashicons-art', 100
            );
        });

        add_action('admin_notices', function () {
            if (!get_option('ezoptimize_key', true)) {
                ?>
                <div class="notice notice-success is-dismissible">
                    <p><?php _e('EZOptimize Setup Needed: Please visit the plugin page for instructions on how to get the API key!',
                            'ezoptimize'); ?></p>
                </div>
                <?php
            }

            if (get_option('ezoptimize', true)) {
                $ezOptimize = get_option('ezoptimize', true);
                if (!empty($ezOptimize->queue)) {
                    ?>
                    <div class="notice notice-success is-dismissible">
                        <p><?php _e('EZOptimize: There are ' . count($ezOptimize->queue) . ' images in the queue. Please visit the plugin page to optimize them!',
                                'ezoptimize'); ?></p>
                    </div>
                    <?php
                }
                //var_dump($ezOptimize->queue);
            }
        });

        add_action('add_attachment', function ($id) {
            if (wp_attachment_is_image($id)) {
                //Maybe directly run reindexing of the files
            }
        });

        $ezData = get_option('ezoptimize', ['queue' => [], 'optimized' => []]);
        if (empty($ezData->queue)) {
            $this->reindexQueue();
        }
    }

    private function ezOptimize($apiKey, $path)
    {
        if (file_exists($path)) {
            $ezOptimize = get_option('ezoptimize', true);

            $originalSize = filesize($path);
            //1.Do a backup
            copy($path, $path . '.uncompressed');
            //2.Get the optimized file

            $args = array(
                'body' => [
                    'source' => base64_encode(file_get_contents(realpath($path))),
                    'apikey' => $apiKey
                ],
                'timeout' => '65',
                'redirection' => '5',
                'httpversion' => '1.0',
                'blocking' => true,
                'headers' => [
                ],
                'cookies' => []
            );

            $httpResponse = wp_remote_post('https://ezoptimize.com/api/optimize', $args);

            if (is_wp_error($httpResponse)) {
                return ['state' => 'error', 'message' => $httpResponse->get_error_message()];
            }

            $response = $this->isJSON($httpResponse['body']) ? json_decode($httpResponse['body']) : false;
            if ($response) {
                if ($response->state == 'success') {
                    $optimizedImage = base64_decode($response->image);
                    $bitesWritten = file_put_contents($path, $optimizedImage);
                    $optimizedSize = $bitesWritten;
                    $ezOptimize->optimized[$path] = (object)[
                        'path' => $path,
                        'name' => pathinfo($path, PATHINFO_BASENAME),
                        'size' => $originalSize,
                        'sizeAfter' => $optimizedSize
                    ];
                    //Remove from queue
                    $queue = $this->reindexQueue();
                    $ezOptimize->queue = $queue->queue;
                    $ezOptimize->optimized = $queue->optimized;
                    update_option('ezoptimize', $ezOptimize);
                    return ['state' => 'success', 'vueData' => $ezOptimize];
                }
                return $response;
            }
        }
        return ['state' => 'error', 'message' => 'Unknown error'];
    }

    private function reindexQueue()
    {
        $filesStorage = $this->getFiles();
        $optimized = [];
        $queue = [];
        foreach ($filesStorage as $item) {
            $item->sizeAfter = filesize($item->path) / 1024;
            if (file_exists($item->path . '.uncompressed')) {
                $item->size = filesize($item->path . '.uncompressed') / 1024;
                $optimized[] = $item;
            } else {
                $queue[] = $item;
            }
        }
        update_option('ezoptimize', (object)['queue' => $queue, 'optimized' => $optimized]);
        return (object)['queue' => $queue, 'optimized' => $optimized];
    }

    public function loadAssets($hook)
    {
        if (is_admin() && strpos($this->menuItem, str_replace(".php", "", $hook)) !== false) {
            wp_enqueue_script('vue', $this->getPluginurl() . '/assets/js/vue.min.js', array('jquery'), '2.4.4', true);
            wp_enqueue_script('swal', $this->getPluginurl() . '/assets/js/sweetalert2.all.min.js', array('jquery'),
                '1.0.0', true);
            wp_enqueue_script('ImageOptimizer', $this->getPluginurl() . '/assets/js/imageoptimizer.js',
                array('jquery', 'vue', 'swal'), '2.0', true);
            wp_enqueue_script('PaceJS', $this->getPluginurl() . '/assets/js/pace.min.js',
                array('jquery', 'vue', 'ImageOptimizer'), '1.0.2', true);
            //Accordion Resources
            wp_enqueue_style('ImageOptimizer', $this->getPluginurl() . '/assets/css/imageoptimizer.css', [], '2.0');
            wp_enqueue_style('PaceCSS',
                $this->getPluginurl() . '/assets/css/pace/themes/blue/pace-theme-corner-indicator.css', [], '1.0.2');
        }
    }

    public function getFiles()
    {
        $ezOptimize = get_option('ezoptimize', true);

        $filesStorage = [];
        $wpUploadData = wp_upload_dir();

        // Get the files we support
        $files = new RegexIterator(new RecursiveIteratorIterator(
            new RecursiveDirectoryIterator($wpUploadData['basedir'], RecursiveDirectoryIterator::SKIP_DOTS),
            RecursiveIteratorIterator::SELF_FIRST,
            RecursiveIteratorIterator::CATCH_GET_CHILD // Ignore "Permission denied"
        ), '/^.+\.(jpg|jpeg|png|svg|gif)$/i', RecursiveRegexIterator::MATCH,
            RecursiveRegexIterator::USE_KEY);

        /** @var SplFileInfo $file */
        foreach ($files as $file) {
            // We only want valid files we can manipulate
            if (!$file->isFile() || !$file->isReadable() || !$file->isWritable()) {
                continue;
            }

            // Get real path as a variable
            $filePath = $file->getRealPath();

            // Not optimized
            if (!isset($ezOptimize->optimized[$filePath])) {
                $filesStorage[$filePath] = (object)[
                    'url' => str_replace($wpUploadData['basedir'], $wpUploadData['baseurl'], $filePath),
                    'state' => false,
                    'path' => $filePath,
                    'name' => $file->getBasename(),
                    'size' => $file->getSize() / 1024 //KB
                ];
            }
        }

        return $filesStorage;
    }

    public function getPluginpath()
    {
        return $this->plugin_path;
    }

    public function getPluginurl()
    {
        return $this->plugin_url;
    }
}

$test = EZO_EZOptimize::get_instance();
$test->getFiles();
